package hr.genetics.models;

import hr.genetics.models.GenericVechileModel.Action;
import hr.seminar.SimulationWriter;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.Random;

public class Nature<T extends GenericVechileModel> {

	private Path environment;
	private Class<T> vechileClass;
	
	private int generationNo = 0;
	
	public List<String> logs = new LinkedList<>();
	
	private Population<GenericVechileModel> population;
	private int populationSize;
	private double mutationPossibility = 0;
	private SimulationWriter simulationWriter = new SimulationWriter();
	
	private List<Simulator> lastSimulations = null;
		
	public Nature(Class<T> vechileClass, int populationSize) {
		
		this.vechileClass = vechileClass;
		this.population = new Population<GenericVechileModel>();
		this.populationSize = populationSize;
		
		this.generateInitialPopulation();
		
	}
	
	private void generateInitialPopulation() {
		
		while (this.population.size() < this.populationSize) {
			try {
				this.population.add(this.vechileClass.newInstance());
			} catch (Exception e) {
				System.out.println(e.getStackTrace());
			}
		}
		
		logs.add("* Initial population successfully born.");
		
	}
	
	public void startEvolution(int maxGenerations) {
		
		while (this.generationNo < maxGenerations) {
			this.testPopulation();
			
			Population<GenericVechileModel> selected = this.doSelection();
			this.breedGeneration(selected);
			this.population = selected;
						
			this.generationNo++;			
		}
		
		simulationWriter.writeStatistics();
		
	}
	
	private void testPopulation() {
		
		logs.add("* Testing generation " + this.generationNo + " started...");
		
		this.lastSimulations = new ArrayList<>();
		
		int sum = 0;
		Simulator bestSimulation = null;
		Simulator currentSimulator = null;
		List<GenericVechileModel> vechiles = this.population;
		for (int i = 0; i < vechiles.size(); ++i) {
			currentSimulator = new Simulator(vechiles.get(i), this.environment);
			this.lastSimulations.add(currentSimulator);
			if (bestSimulation == null || bestSimulation.getScore() < currentSimulator.getScore()) {
				bestSimulation = currentSimulator;
			}
			sum += bestSimulation.getScore();
		}
		
		logs.add("* Testing generation " + this.generationNo + " ended.");
		
		logs.add("* Avg score: " + sum/vechiles.size()
				+ ", best score: " + bestSimulation.getScore()
				+ ", last position: " + bestSimulation.getSimulationModel().getPosition()
				+ ", passed checkpoints: " + bestSimulation.getSimulationModel().passedCheckpoints()
		);
		
		simulationWriter.setFilenamePrefix("gen" + this.generationNo + "_");
		simulationWriter.setPathID(this.environment.getId());
		String stopReason = null;
		if (bestSimulation.getSimulationModel().isCrashed())
			stopReason = "CRASHED";
		if (bestSimulation.getSimulationModel().isFinished())
			stopReason = "FINISHED";
		simulationWriter.setStopReason(stopReason);
		simulationWriter.writeSimilation(bestSimulation.getReplay());
		simulationWriter.addStats(sum/vechiles.size(), bestSimulation.getScore());
				
	}
	
	public void setEnvironment(Path environment) {
		this.environment = environment;
	}
	
	/**
	 * tournament selection
	 * @return selected generation (half of whole)
	 */
	private Population<GenericVechileModel> doSelection() {
		
	    Random generator = new Random();
		Population<GenericVechileModel> selected = new Population<>();
		int size = this.population.size();
		
		while (selected.size() < size/2) {
			Simulator first = this.lastSimulations.get(generator.nextInt(size)),
						second = this.lastSimulations.get(generator.nextInt(size)),
						third = this.lastSimulations.get(generator.nextInt(size)),
						fourth = this.lastSimulations.get(generator.nextInt(size)),
						firstWinner = null,
						secondWinner = null;
			
			Collections.shuffle(this.lastSimulations);
						
			// first match
			if (first.getScore() > second.getScore()) {
				firstWinner = first;
			} else {
				firstWinner = second;
			}
			
			// second match
			if (third.getScore() > fourth.getScore()) {
				secondWinner = third;
			} else {
				secondWinner = fourth;
			}
			
			// final round
			if (firstWinner.getScore() > secondWinner.getScore()) {
				selected.add(firstWinner.getSimulationModel().getVechileModel());
			} else {
				selected.add(secondWinner.getSimulationModel().getVechileModel());
			}
			
		}
		
		return selected;
		
	}
	
	private void breedGeneration(Population<GenericVechileModel> vechiles) {
		
		Random generator = new Random();
		int parentsNo = vechiles.size();
				
		logs.add("* Started breeding generation "+this.generationNo);
		
		while (vechiles.size() < this.population.size()) {
			GenericVechileModel first = vechiles.get(generator.nextInt(parentsNo));
			GenericVechileModel second = vechiles.get(generator.nextInt(parentsNo));
			
			vechiles.add(this.breed(first, second));
		}
		
		logs.add("* Breeding generation "+this.generationNo+" finished.");
				
	}
	
	private T breed(GenericVechileModel first, GenericVechileModel second) {
		
		T newVechile = null;
		
		try {
			GenericVechileModel.Action[] firstDNA = first.getDNA();
			GenericVechileModel.Action[] secondDNA = second.getDNA();
			
			newVechile = this.vechileClass.newInstance();
			newVechile.setDimensions(first.getDimensions());
			newVechile.setDNASize(first.getDNASize());
			newVechile.setDNA(this.combineDNA(firstDNA, secondDNA));
		} catch (Exception e) {
			System.out.println(e.getStackTrace());
		}
		
		this.mutate(newVechile);
				
		return newVechile; 
		
	}
	
	private void mutate(GenericVechileModel single) {

		GenericVechileModel.Action[] DNA = single.getDNA();
		
		Random pick = new Random();
		int actionsTotal = Action.values().length;
		List<Action> values = Arrays.asList(Action.values());
		for (int i = 0; i < DNA.length; ++i) {
			if (Math.random() <= this.mutationPossibility) {
				int random = pick.nextInt(actionsTotal);
				DNA[i] = values.get(random);
			}
		}
		
	}
	
	private GenericVechileModel.Action[] combineDNA(GenericVechileModel.Action[] first,
			GenericVechileModel.Action[] second) {
		
		GenericVechileModel.Action[] newDNA = new GenericVechileModel.Action[first.length];
		int breakPosition = new Random().nextInt(first.length);
		
		for (int i = 0; i < breakPosition; ++i) {
			newDNA[i] = first[i];
		}
		
		for (int i = breakPosition; i < first.length; ++i) {
			newDNA[i] = second[i];
		}
		
		return newDNA;
		
	}

	/**
	 * @return the mutationPossibility
	 */
	public double getMutationPossibility() {
		return mutationPossibility;
	}

	/**
	 * @param mutationPossibility the mutationPossibility to set
	 */
	public void setMutationPossibility(double mutationPossibility) {
		this.mutationPossibility = mutationPossibility;
	}
	
}
